# -*- coding: utf-8 -*-

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from maintenance_mode.io import read_file, write_file


def get_maintenance_mode():
    """
    Get maintenance_mode state from state file.
    """

    # If maintenance mode is defined in settings, it has priority.
    if settings.MAINTENANCE_MODE is not None:
        return settings.MAINTENANCE_MODE

    value = read_file(settings.MAINTENANCE_MODE_STATE_FILE_PATH, '0')

    if value not in ['0', '1']:
        raise ValueError('state file content value is not 0|1')

    value = bool(int(value))
    return value


def set_maintenance_mode(value):
    """
    Set maintenance_mode state to state file.
    """

    # If maintenance mode is defined in settings, it can't be changed.
    if settings.MAINTENANCE_MODE is not None:
        raise ImproperlyConfigured(
            'Maintenance mode cannot be set dynamically ' \
            'if defined in settings.')

    if not isinstance(value, bool):
        raise TypeError('value argument type is not boolean')

    value = str(int(value))
    write_file(settings.MAINTENANCE_MODE_STATE_FILE_PATH, value)



# -*- coding: utf-8 -*-

import django

if django.VERSION < (1, 10):
    __MaintenanceModeMiddlewareBaseClass = object
else:
    # https://docs.djangoproject.com/en/1.10/topics/http/middleware/#upgrading-pre-django-1-10-style-middleware
    from django.utils.deprecation import MiddlewareMixin
    __MaintenanceModeMiddlewareBaseClass = MiddlewareMixin

from maintenance_mode.http import (
    get_maintenance_response, need_maintenance_response, )


class MaintenanceModeMiddleware(__MaintenanceModeMiddlewareBaseClass):

    def process_request(self, request):
        if need_maintenance_response(request):
            return get_maintenance_response(request)
        else:
            return None



# -*- coding: utf-8 -*-

import django
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

if django.VERSION < (2, 0):
    from django.core.urlresolvers import (
        NoReverseMatch, resolve, Resolver404, reverse, )
else:
    from django.urls import (
        NoReverseMatch, resolve, Resolver404, reverse, )

from django.shortcuts import render, redirect
from django.template import RequestContext
from django.utils.cache import add_never_cache_headers
from django.utils.module_loading import import_string

from maintenance_mode.core import get_maintenance_mode
from maintenance_mode.utils import get_client_ip_address

import re
import sys


def get_maintenance_response(request):
    """
    Return a '503 Service Unavailable' maintenance response.
    """
    if settings.MAINTENANCE_MODE_REDIRECT_URL:
        return redirect(settings.MAINTENANCE_MODE_REDIRECT_URL)

    context = {}

    if settings.MAINTENANCE_MODE_GET_TEMPLATE_CONTEXT:
        try:
            get_request_context_func = import_string(
                settings.MAINTENANCE_MODE_GET_TEMPLATE_CONTEXT)
        except ImportError:
            raise ImproperlyConfigured(
                'settings.MAINTENANCE_MODE_GET_TEMPLATE_CONTEXT '
                'is not a valid function path.'
            )

        context = get_request_context_func(request=request)

    if django.VERSION < (1, 8):
        kwargs = {'context_instance': RequestContext(request, context)}
    else:
        kwargs = {'context': context}

    response = render(request, settings.MAINTENANCE_MODE_TEMPLATE,
                      content_type='text/html', status=503, **kwargs)

    add_never_cache_headers(response)
    return response


def need_maintenance_response(request):
    """
    Tells if the given request needs a maintenance response or not.
    """

    try:
        view_match = resolve(request.path)
        view_func = view_match[0]
        view_dict = view_func.__dict__

        view_force_maintenance_mode_off = view_dict.get(
            'force_maintenance_mode_off', False)
        if view_force_maintenance_mode_off:
            # view has 'force_maintenance_mode_off' decorator
            return False

        view_force_maintenance_mode_on = view_dict.get(
            'force_maintenance_mode_on', False)
        if view_force_maintenance_mode_on:
            # view has 'force_maintenance_mode_on' decorator
            return True

    except Resolver404:
        pass

    if not get_maintenance_mode():
        return False

    try:
        url_off = reverse('maintenance_mode_off')

        resolve(url_off)

        if url_off == request.path_info:
            return False

    except NoReverseMatch:
        # maintenance_mode.urls not added
        pass

    if hasattr(request, 'user'):

        if django.VERSION < (1, 10):
            if settings.MAINTENANCE_MODE_IGNORE_ANONYMOUS_USER \
                    and request.user.is_anonymous():
                return False

            if settings.MAINTENANCE_MODE_IGNORE_AUTHENTICATED_USER \
                    and request.user.is_authenticated():
                return False
        else:
            if settings.MAINTENANCE_MODE_IGNORE_ANONYMOUS_USER \
                    and request.user.is_anonymous:
                return False

            if settings.MAINTENANCE_MODE_IGNORE_AUTHENTICATED_USER \
                    and request.user.is_authenticated:
                return False

        if settings.MAINTENANCE_MODE_IGNORE_STAFF \
                and request.user.is_staff:
            return False

        if settings.MAINTENANCE_MODE_IGNORE_SUPERUSER \
                and request.user.is_superuser:
            return False

    if settings.MAINTENANCE_MODE_IGNORE_TESTS:

        is_testing = False

        if (len(sys.argv) > 0 and 'runtests' in sys.argv[0]) \
                or (len(sys.argv) > 1 and sys.argv[1] == 'test'):
            # python runtests.py | python manage.py test | python
            # setup.py test | django-admin.py test
            is_testing = True

        if is_testing:
            return False

    if settings.MAINTENANCE_MODE_IGNORE_IP_ADDRESSES:

        if settings.MAINTENANCE_MODE_GET_CLIENT_IP_ADDRESS:
            try:
                get_client_ip_address_func = import_string(
                    settings.MAINTENANCE_MODE_GET_CLIENT_IP_ADDRESS)
            except ImportError:
                raise ImproperlyConfigured(
                    'settings.MAINTENANCE_MODE_GET_CLIENT_IP_ADDRESS '
                    'is not a valid function path.')
            else:
                client_ip_address = get_client_ip_address_func(request)
        else:
            client_ip_address = get_client_ip_address(request)

        for ip_address in settings.MAINTENANCE_MODE_IGNORE_IP_ADDRESSES:

            ip_address_re = re.compile(ip_address)

            if ip_address_re.match(client_ip_address):
                return False

    if settings.MAINTENANCE_MODE_IGNORE_URLS:

        for url in settings.MAINTENANCE_MODE_IGNORE_URLS:

            if not isinstance(url, re._pattern_type):
                url = str(url)
            url_re = re.compile(url)

            if url_re.match(request.path_info):
                return False

    if settings.MAINTENANCE_MODE_REDIRECT_URL:

        redirect_url_re = re.compile(
            settings.MAINTENANCE_MODE_REDIRECT_URL)

        if redirect_url_re.match(request.path_info):
            return False

    return True



# -*- coding: utf-8 -*-

from django.conf import settings
from django.utils.module_loading import import_module

import os


if not hasattr(settings, 'MAINTENANCE_MODE'):
    settings.MAINTENANCE_MODE = None

if not hasattr(settings, 'MAINTENANCE_MODE_GET_CLIENT_IP_ADDRESS'):
    settings.MAINTENANCE_MODE_GET_CLIENT_IP_ADDRESS = None

if not hasattr(settings, 'MAINTENANCE_MODE_GET_TEMPLATE_CONTEXT'):
    settings.MAINTENANCE_MODE_GET_TEMPLATE_CONTEXT = None

if not hasattr(settings, 'MAINTENANCE_MODE_IGNORE_IP_ADDRESSES'):
    settings.MAINTENANCE_MODE_IGNORE_IP_ADDRESSES = None

if not hasattr(settings, 'MAINTENANCE_MODE_IGNORE_ANONYMOUS_USER'):
    settings.MAINTENANCE_MODE_IGNORE_ANONYMOUS_USER = False

if not hasattr(settings, 'MAINTENANCE_MODE_IGNORE_AUTHENTICATED_USER'):
    settings.MAINTENANCE_MODE_IGNORE_AUTHENTICATED_USER = False

if not hasattr(settings, 'MAINTENANCE_MODE_IGNORE_STAFF'):
    settings.MAINTENANCE_MODE_IGNORE_STAFF = False

if not hasattr(settings, 'MAINTENANCE_MODE_IGNORE_SUPERUSER'):
    settings.MAINTENANCE_MODE_IGNORE_SUPERUSER = False

if not hasattr(settings, 'MAINTENANCE_MODE_IGNORE_TESTS'):
    settings.MAINTENANCE_MODE_IGNORE_TESTS = False

if not hasattr(settings, 'MAINTENANCE_MODE_IGNORE_URLS'):
    settings.MAINTENANCE_MODE_IGNORE_URLS = None

if not hasattr(settings, 'MAINTENANCE_MODE_REDIRECT_URL'):
    settings.MAINTENANCE_MODE_REDIRECT_URL = None

if not hasattr(settings, 'MAINTENANCE_MODE_STATE_FILE_NAME'):
    settings.MAINTENANCE_MODE_STATE_FILE_NAME = 'maintenance_mode_state.txt'

if not hasattr(settings, 'MAINTENANCE_MODE_STATE_FILE_PATH'):
    settings_module = import_module(os.environ['DJANGO_SETTINGS_MODULE'])
    settings_path = settings_module.__file__
    settings_dir = os.path.dirname(settings_path)
    settings.MAINTENANCE_MODE_STATE_FILE_PATH = os.path.abspath(
        os.path.join(os.sep, settings_dir,
            settings.MAINTENANCE_MODE_STATE_FILE_NAME))

if not hasattr(settings, 'MAINTENANCE_MODE_TEMPLATE'):
    settings.MAINTENANCE_MODE_TEMPLATE = '503.html'



# -*- coding: utf-8 -*-

import django

if django.VERSION < (2, 0):
    from django.conf.urls import url as re_path
else:
    from django.urls import re_path

from maintenance_mode.views import maintenance_mode_off, maintenance_mode_on


urlpatterns = [
    re_path(r'^off/$', maintenance_mode_off, name='maintenance_mode_off'),
    re_path(r'^on/$', maintenance_mode_on, name='maintenance_mode_on'),
]



# -*- coding: utf-8 -*-

import os


def read_file(file_path, default_content=''):
    """
    Read file at the specified path.
    If file doesn't exist, it will be created with default-content.
    Returns the file content.
    """
    if not os.path.exists(file_path):
        write_file(file_path, default_content)

    handler = open(file_path, 'r')
    content = handler.read()
    handler.close()
    return content or default_content


def write_file(file_path, content):
    """
    Write file at the specified path with content.
    If file exists, it will be overwritten.
    """
    handler = open(file_path, 'w+')
    handler.write(content)
    handler.close()



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import django

from django.core.management.base import BaseCommand, CommandError

from maintenance_mode import core


class Command(BaseCommand):

    args = '<on|off>'
    help = 'run python manage.py maintenance_mode %s '\
           'to change maintenance-mode state' % args

    def add_arguments(self, parser):
        parser.add_argument('state')

    def confirm(self, message):
        # Fix for Python 2.x.
        try:
            input_func = raw_input
        except NameError:
            input_func = input

        answer = input_func(message)
        answer = answer.lower()
        return answer.find('y') == 0

    def handle(self, *args, **options):

        verbosity = int(options['verbosity'])
        verbose = True if verbosity == 3 else False

        if django.VERSION < (1, 8):
            if len(args) != 1:
                raise CommandError(
                    'Error: expected 1 argument: %s' % (self.args, ))

            state = args[0]
        else:
            state = options['state']

        state = state.lower()
        value = core.get_maintenance_mode()

        if state in ['on', 'yes', 'true', '1']:

            if value:
                if verbose:
                    self.stdout.write('maintenance mode is already on')
                return

            if verbose:
                if self.confirm('maintenance mode on? (y/N) '):
                    core.set_maintenance_mode(True)
            else:
                core.set_maintenance_mode(True)

        elif state in ['off', 'no', 'false', '0']:

            if not value:
                if verbose:
                    self.stdout.write('maintenance mode is already off')
                return

            if verbose:
                if self.confirm('maintenance mode off? (y/N) '):
                    core.set_maintenance_mode(False)
            else:
                core.set_maintenance_mode(False)

        else:
            raise CommandError('Error: invalid argument: \'%s\' '
                               'expected %s' % (state, self.args, ))

        if verbose:
            output = 'maintenance mode: %s' % (
                'on' if core.get_maintenance_mode() else 'off', )
            self.stdout.write(output)

        return



# -*- coding: utf-8 -*-

from maintenance_mode.http import get_maintenance_response


def force_maintenance_mode_off(view_func):
    def wrap(request, *args, **kwargs):
        return view_func(request, *args, **kwargs)
    wrap.__dict__['force_maintenance_mode_off'] = True
    wrap.__doc__ = view_func.__doc__
    wrap.__name__ = view_func.__name__
    return wrap


def force_maintenance_mode_on(view_func):
    def wrap(request, *args, **kwargs):
        return get_maintenance_response(request)
    wrap.__dict__['force_maintenance_mode_on'] = True
    wrap.__doc__ = view_func.__doc__
    wrap.__name__ = view_func.__name__
    return wrap



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import os

exec(open('maintenance_mode/version.py').read())

github_url = 'https://github.com/fabiocaccamo'
package_name = 'django-maintenance-mode'
package_path = os.path.abspath(os.path.dirname(__file__))
long_description_file_path = os.path.join(package_path, 'README.rst')
long_description = ''
try:
    with open(long_description_file_path) as f:
        long_description = f.read()
except IOError:
    pass

setup(
    name=package_name,
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    version=__version__,
    description='django-maintenance-mode shows a 503 error page when maintenance-mode is on.',
    long_description=long_description,
    author='Fabio Caccamo',
    author_email='fabio.caccamo@gmail.com',
    url='%s/%s' % (github_url, package_name, ),
    download_url='%s/%s/archive/%s.tar.gz' % (github_url, package_name, __version__, ),
    keywords=['django', 'maintenance', 'mode', 'offline', 'under', '503', 'service', 'temporarily', 'unavailable'],
    requires=['django(>=1.7)'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Build Tools',
    ],
    license='MIT',
    test_suite='runtests.runtests'
)
